/***********************************************
*                                              *
*        Jeff Molofee's OpenGL Basecode        *
*                                              *
*            http://nehe.gamedev.net           *
*                                              *
*              Visual C++ 6.0 2002             *
*                                              *
*          Recompil par Bob Nol 2018         *
*              Visual Studio 2017              *
*                                              *
***********************************************/

#include <windows.h>													// Fichier d'entte pour Windows
#include <gl/gl.h>														// Fichier d'entte pour OpenGL32
#include <gl/glu.h>														// Fichier d'entte pour GLu32

#include "NeHeGL.h"														// Fichier d'entte pour le code de base de la fentre GL

#pragma warning(disable : 4838 )						// La conversion de 'int'  'BYTE' requiert une conversion restrictive

#define WM_TOGGLEFULLSCREEN (WM_USER+1)									// Dfinir le message pour basculer
#define IDI_ICON1
#define IDI_ICON2
																		// entre le mode plein cran / fentre
static BOOL g_isProgramLooping;											// Boucle de cration de fentre, pour le passage 
																		// plein cran / fentre
static BOOL g_createFullScreen;											// Si VRAI, crer en plein cran

void TerminateApplication (GL_Window* window)							// Terminer l'application
{
	PostMessage (window->hWnd, WM_QUIT, 0, 0);							// Envoyer un message WM_QUIT
	g_isProgramLooping = FALSE;											// Arrter la boucle du programme
}

void ToggleFullscreen (GL_Window* window)								// Basculer en plein cran / Fentre
{
	PostMessage (window->hWnd, WM_TOGGLEFULLSCREEN, 0, 0);				// Envoyer un message WM_TOGGLEFULLSCREEN
}

void ReshapeGL (int width, int height)									// Remodeler la fentre lorsqu'elle est dplace 
																		// ou redimensionne
{
	glViewport (0, 0, (GLsizei)(width), (GLsizei)(height));				// Rinitialiser la fentre d'affichage actuelle
	glMatrixMode (GL_PROJECTION);										// Slectionnez la matrice de projection
	glLoadIdentity ();													// Rinitialiser la matrice de projection
	gluPerspective (45.0f, (GLfloat)(width)/(GLfloat)(height),			// Calculer le format de la fentre
					1.0f, 100.0f);		
	glMatrixMode (GL_MODELVIEW);										// Slectionnez la matrice Modelview
	glLoadIdentity ();													// Rinitialiser la matrice Modelview
}

BOOL ChangeScreenResolution (int width, int height, int bitsPerPixel)	// Changer la rsolution de l'cran
{
	DEVMODE dmScreenSettings;											// Mode appareil
	ZeroMemory (&dmScreenSettings, sizeof (DEVMODE));					// Assurez-vous que la mmoire est vide
	dmScreenSettings.dmSize				= sizeof (DEVMODE);				// Taille de la structure du Devmode
	dmScreenSettings.dmPelsWidth		= width;						// Slectionnez la largeur de l'cran
	dmScreenSettings.dmPelsHeight		= height;						// Slectionnez la hauteur de l'cran
	dmScreenSettings.dmBitsPerPel		= bitsPerPixel;					// Slectionner bits par pixel
	dmScreenSettings.dmFields			= DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
	if (ChangeDisplaySettings (&dmScreenSettings, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
	{
		return FALSE;													// chec de la modification d'affichage, retourne False
	}
	return TRUE;														// Le changement d'affichage a russi, retourne True
}

BOOL CreateWindowGL (GL_Window* window)									// Ce code cre notre fentre OpenGL
{
	DWORD windowStyle = WS_OVERLAPPEDWINDOW;							// Dfinir notre style de fentre
	DWORD windowExtendedStyle = WS_EX_APPWINDOW;						// Dfinir le style tendu de la fentre

	PIXELFORMATDESCRIPTOR pfd =											// pfd indique  Windows comment nous voulons que les 
																		// choses soient
	{
		sizeof (PIXELFORMATDESCRIPTOR),									// Taille de ce descripteur de format pixel
		1,																// Numro de version
		PFD_DRAW_TO_WINDOW |											// Le format doit supporter la fentre
		PFD_SUPPORT_OPENGL |											// Le format doit prendre en charge OpenGL
		PFD_DOUBLEBUFFER,												// Doit prendre en charge la double mise en tampon
		PFD_TYPE_RGBA,													// Demander un format RGBA
		window->init.bitsPerPixel,										// Slectionnez notre profondeur de couleur
		0, 0, 0, 0, 0, 0,												// Bits de couleur ignors
		0,																// Pas de tampon alpha
		0,																// Bit de dcalage ignor
		0,																// Pas de tampon d'accumulation
		0, 0, 0, 0,														// Bits d'accumulation ignors
		16,																// Tampon Z 16 bits (tampon de profondeur)  
		0,																// Pas de tampon pochoir
		0,																// Pas de tampon auxiliaire
		PFD_MAIN_PLANE,													// Couche de dessin principale
		0,																// Rserv
		0, 0, 0															// Masques de calque ignors
	};

	RECT windowRect = {0, 0, window->init.width, window->init.height};	// Dfinir nos coordonnes de fentre

	GLuint PixelFormat;													// Tiendra le format de pixel slectionn

	if (window->init.isFullScreen == TRUE)								// Plein cran demand, essayez de changer de mode vido
	{
		if (ChangeScreenResolution (window->init.width, window->init.height, window->init.bitsPerPixel) == FALSE)
		{
			// Le mode plein cran a chou. Excuter en mode fentr  la place
			MessageBox (HWND_DESKTOP, "chec du changement de mode.\nFonctionne en mode fentre.", 
									  "Erreur", MB_OK | MB_ICONEXCLAMATION);
			window->init.isFullScreen = FALSE;							// Dfinir le mode plein cran sur False (mode fentre)
		}
		else															// Sinon (si le mode plein cran a russi)
		{
			ShowCursor (FALSE);											// teindre le curseur
			windowStyle = WS_POPUP;										// Dfinissez le WindowStyle sur WS_POPUP 
																		// (fentre contextuelle)
			windowExtendedStyle |= WS_EX_TOPMOST;						// Dfinissez le WindowStyle tendu sur WS_EX_TOPMOST
		}																// (La fentre du haut couvre tout le reste)
	}
	else																// Si le plein cran n'a pas t slectionn
	{
		// Ajuster la fentre, compte pour les bordures de fentre
		AdjustWindowRectEx (&windowRect, windowStyle, 0, windowExtendedStyle);
	}

	// Crer la fentre OpenGL
	window->hWnd = CreateWindowEx (windowExtendedStyle,					// Style tendu
								   window->init.application->className,	// Nom de la classe
								   window->init.title,					// Titre de la fentre
								   windowStyle,							// Style de fentre
								   0, 0,								// Position de la fentre X,Y
								   windowRect.right - windowRect.left,	// Largeur de la fentre
								   windowRect.bottom - windowRect.top,	// Hauteur de la fentre
								   HWND_DESKTOP,						// Le bureau est le parent de la fentre
								   0,									// Pas de menu
								   window->init.application->hInstance, // Passer l'instance de la fentre
								   window);

	if (window->hWnd == 0)												// La cration de fentres a-t-elle t un succs?
	{
		return FALSE;													// Si non retourn False
	}

	window->hDC = GetDC (window->hWnd);									// Saisir un contexte de priphrique pour cette fentre
	if (window->hDC == 0)												// Avons-nous obtenu un contexte de priphrique?
	{
		// chou
		DestroyWindow (window->hWnd);									// Dtruire la fentre
		window->hWnd = 0;												// Zro le handle de la fentre
		return FALSE;													// Retourne False
	}

	PixelFormat = ChoosePixelFormat (window->hDC, &pfd);				// Trouver un format de pixel compatible
	if (PixelFormat == 0)												// Avons-nous trouv un format compatible?
	{
		// chou
		ReleaseDC (window->hWnd, window->hDC);							// Librer notre contexte de priphrique
		window->hDC = 0;												// Zro le contexte de l'appareil
		DestroyWindow (window->hWnd);									// Dtruire la fentre
		window->hWnd = 0;												// Zro le handle de la fentre
		return FALSE;													// Retourne False
	}

	if (SetPixelFormat (window->hDC, PixelFormat, &pfd) == FALSE)		// Essayez de dfinir le format de pixel
	{
		// chou
		ReleaseDC (window->hWnd, window->hDC);							// Librer notre contexte de priphrique
		window->hDC = 0;												// Zro le contexte de l'appareil
		DestroyWindow (window->hWnd);									// Dtruire la fentre
		window->hWnd = 0;												// Zro le handle de la fentre
		return FALSE;													// Retourne False
	}

	window->hRC = wglCreateContext (window->hDC);						// Essayer d'obtenir un contexte de rendu
	if (window->hRC == 0)												// Avons-nous obtenu un contexte de rendu?
	{
		// chou
		ReleaseDC (window->hWnd, window->hDC);							// Librer notre contexte de priphrique
		window->hDC = 0;												// Zro le contexte de l'appareil
		DestroyWindow (window->hWnd);									// Dtruire la fentre
		window->hWnd = 0;												// Zro le handle de la fentre
		return FALSE;													// Retourne False
	}

	// Faire du contexte de rendu notre contexte de rendu actuel
	if (wglMakeCurrent (window->hDC, window->hRC) == FALSE)
	{
		// chou
		wglDeleteContext (window->hRC);									// Supprimer le contexte de rendu
		window->hRC = 0;												// Zro le contexte de rendu
		ReleaseDC (window->hWnd, window->hDC);							// Librer notre contexte de priphrique
		window->hDC = 0;												// Zro le contexte de l'appareil
		DestroyWindow (window->hWnd);									// Dtruire la fentre
		window->hWnd = 0;												// Zro le handle de la fentre
		return FALSE;													// Retourne False
	}

	ShowWindow (window->hWnd, SW_NORMAL);								// Rendre la fentre visible
	window->isVisible = TRUE;											// Mettre l'ensemble visible  True

	ReshapeGL (window->init.width, window->init.height);				// Remodeler notre fentre GL

	ZeroMemory (window->keys, sizeof (Keys));							// Effacer toutes les cls

	window->lastTickCount = GetTickCount ();							// Obtenez le nombre de tiques

	return TRUE;														// La cration de fentre a t un succs
																		// L'initialisation sera faite dans WM_CREATE
}

BOOL DestroyWindowGL (GL_Window* window)								// Dtruire la fentre OpenGL et librer les ressources
{
	if (window->hWnd != 0)												// La fentre a-t-elle un Handle?
	{	
		if (window->hDC != 0)											// La fentre a-t-elle un contexte de priphrique?
		{
			wglMakeCurrent (window->hDC, 0);							// Dfinir le contexte de rendu actif actuel sur zro
			if (window->hRC != 0)										// La fentre a-t-elle un contexte de rendu?
			{
				wglDeleteContext (window->hRC);							// Librer le contexte de rendu
				window->hRC = 0;										// Contexte de rendu  zro

			}
			ReleaseDC (window->hWnd, window->hDC);						// Librer le contexte de rendu
			window->hDC = 0;											// Contexte de rendu  zro
		}
		DestroyWindow (window->hWnd);									// Dtruire la fentre
		window->hWnd = 0;												// Le handle de fentre  zro
	}

	if (window->init.isFullScreen)										// La fentre est-elle en mode plein cran?
	{
		ChangeDisplaySettings (NULL,0);									// Revenir  la rsolution du bureau
		ShowCursor (TRUE);												// Montrer le curseur
	}	
	return TRUE;														// Retourne True
}

// Rappels de messages de la fentre de processus
LRESULT CALLBACK WindowProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	DWORD				tickCount;										// Utilis pour le compteur de tiques

	// Obtenir le contexte de la fentre
	GL_Window* window = (GL_Window*)(GetWindowLong (hWnd, GWL_USERDATA));

	switch (uMsg)														// Evaluer le message de la fentre
	{
		case WM_PAINT:													// La fentre doit tre mise  jour
		{
			tickCount = GetTickCount ();								// Obtenez le nombre de tiques
			Update (tickCount - window->lastTickCount);					// Mettre  jour le compteur
			window->lastTickCount = tickCount;							// Dfinir le dernier compte sur le nombre actuel
			Draw ();													// Dessine notre scne
			SwapBuffers (window->hDC);									// changer les tampons (Double mmoire tampon)
		}
		return 0;														// Retourne 0

		case WM_SYSCOMMAND:												// intercepte les commandes du systme
		{
			switch (wParam)												// Vrifier les appels systme
			{
				case SC_SCREENSAVE:										// Est-ce que l'cran de veille dmarre?
				case SC_MONITORPOWER:									// Est-ce que l'ecran tente l'economie d'energie?
				return 0;												// On empche que cela se passe
			}
			break;														// On sort
		}
		return 0;														// Retourne 0

		case WM_CREATE:													// Cration de la fentre
		{
			CREATESTRUCT* creation = (CREATESTRUCT*)(lParam);			// Pointeur de structure fentre
			window = (GL_Window*)(creation->lpCreateParams);
			SetWindowLong (hWnd, GWL_USERDATA, (LONG)(window));
		}
		return 0;														// Retourne 0

		case WM_CLOSE:													// Fermer la fentre
			TerminateApplication(window);								// Terminer l'application
		return 0;														// Retourne 0

		case WM_SIZE:													// L'action de taille a pris place
			switch (wParam)												// valuer l'action de taille
			{
				case SIZE_MINIMIZED:									// La fentre a-t-elle t minimise?
					window->isVisible = FALSE;							// Mettre l'ensemble visible  False
				return 0;												// Retourne 0

				case SIZE_MAXIMIZED:									// La fentre a-t-elle t maximise?
					window->isVisible = TRUE;							// Mettre l'ensemble visible  True
					ReshapeGL (LOWORD (lParam), HIWORD (lParam));		// Remodeler la fentre - LoWord=Largeur, HiWord=Hauteur
				return 0;												// Retourne 0

				case SIZE_RESTORED:										// La fentre a-t-elle t restaure?
					window->isVisible = TRUE;							// Mettre l'ensemble visible  True
					ReshapeGL (LOWORD (lParam), HIWORD (lParam));		// Remodeler la fentre - LoWord=Largeur, HiWord=Hauteur
				return 0;												// Retourne 0
			}
		break;															// Pause

		case WM_KEYDOWN:												// Mettre  jour les tampons de clavier pour les touches 
																		// presses
			if ((wParam >= 0) && (wParam <= 255))						// La touche (wParam) est-elle dans une plage valide?
			{
				window->keys->keyDown [wParam] = TRUE;					// Dfinissez la touche slectionne (wParam) sur True
				return 0;												// Retourne 0
			}
		break;															// Pause

		case WM_KEYUP:													// Mise  jour des tampons de clavier pour les touches 
																		// relaches
			if ((wParam >= 0) && (wParam <= 255))						// La touche (wParam) est-elle dans une plage valide?
			{
				window->keys->keyDown [wParam] = FALSE;					// Dfinissez la touche slectionne (wParam) sur False
				return 0;												// Retourne 0
			}
		break;															// Pause

		case WM_TOGGLEFULLSCREEN:										// Activer / dsactiver le mode plein cran
			g_createFullScreen = (g_createFullScreen == TRUE) ? FALSE : TRUE;
			PostMessage (hWnd, WM_QUIT, 0, 0);
		break;															// Pause
	}

	return DefWindowProc (hWnd, uMsg, wParam, lParam);					// Transfrer des messages non grs  DefWindowProc
}

BOOL RegisterWindowClass (Application* application)						// Enregistrer une classe de fentre pour cette application
{																		// TRUE si russi
	// Inscrire une classe fentre
	WNDCLASSEX windowClass;												// Classe fentre
	ZeroMemory (&windowClass, sizeof (WNDCLASSEX));						// Assurez-vous que la mmoire est efface
	windowClass.cbSize			= sizeof (WNDCLASSEX);					// Taille de la structure windowClass
	windowClass.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	// Redessine la fentre pour tout mouvement 
																		// / redimensionnement
	windowClass.lpfnWndProc		= (WNDPROC)(WindowProc);				// WindowProc gre les messages
	windowClass.hInstance		= application->hInstance;				// Dfinir l'instance
	windowClass.hbrBackground	= (HBRUSH)COLOR_WINDOW+1;				// Couleur de fond du pinceau
	windowClass.hIcon			= LoadIcon(application->hInstance, ("Logo"));		// Charge l'icne par dfaut
	windowClass.hIconSm			= LoadIcon(application->hInstance, ("Main"));		// Change l'Icne de la fentre
	windowClass.hCursor			= LoadCursor(NULL, IDC_ARROW);			// Charger le pointeur flche
	windowClass.lpszClassName	= application->className;				// Dfinit le nom de classe application
	if (RegisterClassEx (&windowClass) == 0)							// L'enregistrement de la classe a-t-il chou?
	{
		// REMARQUE: L'chec ne devrait jamais se produire
		MessageBox (HWND_DESKTOP, "L'enregistrement ClassEx a chou!", "Erreur", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;													// Renvoie False (Echec)
	}
	return TRUE;														// Renvoie True (Succs)
}

// Entre de programme (WinMain)
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	Application			application;									// Structure d'application
	GL_Window			window;											// Structure de la fentre
	Keys				keys;											// Structures des touches
	BOOL				isMessagePumpActive;							// Message pompe active?
	MSG					msg;											// Structure message de la fentre

	// Remplir les donnes d'application
	application.className = "OpenGL";									// Nom de la classe application
	application.hInstance = hInstance;									// Instance de l'application

	// Remplir la fentre
	ZeroMemory (&window, sizeof (GL_Window));							// Assurez-vous que la mmoire est mise  zro
	window.keys					= &keys;								// Structure des touches
	window.init.application		= &application;							// Application de fentre

	// Titre de la fentre
	window.init.title			= "Leon 01: Sphre GLU et Cube";

	window.init.width			= 640;									// Largeur de la fentre
	window.init.height			= 480;									// Hauteur de la fentre
	window.init.bitsPerPixel	= 32;									// Bits par pixel
	window.init.isFullScreen	= TRUE;									// Plein cran? (Rgl sur TRUE)

	ZeroMemory (&keys, sizeof (Keys));									// Structure des touches mise  zro

	// Demander  l'utilisateur s'il souhaite dmarrer en mode plein cran?
	if (MessageBox (HWND_DESKTOP, "Souhaitez-vous excuter en mode plein cran?", "Dmarrer plein cran?", 
								  MB_YESNO | MB_ICONQUESTION) == IDNO)
	{
		window.init.isFullScreen = FALSE;								// Si non, excuter en mode fentre
	}

	// Enregistrer une classe  utiliser pour notre fentre
	if (RegisterWindowClass (&application) == FALSE)					// L'enregistrement d'une classe a-t-il chou?
	{
		// chec
		MessageBox (HWND_DESKTOP, "Erreur lors de l'enregistrement de la classe de fentre!", "Erreur", MB_OK | MB_ICONEXCLAMATION);
		return -1;														// Terminer l'application
	}

	g_isProgramLooping = TRUE;											// La boucle du programme est rgl sur TRUE
	g_createFullScreen = window.init.isFullScreen;						// g_createFullScreen est dfini par dfaut par l'utilisateur
	while (g_isProgramLooping)											// Boucle jusqu' WM_QUIT Est reu
	{
		// Crer une fentre
		window.init.isFullScreen = g_createFullScreen;					// Dfinir Init Param De La Cration De La Fentre En Plein 
																		// cran?
		if (CreateWindowGL (&window) == TRUE)							// La cration de fentre a-t-elle russi?
		{
			//  ce stade, nous devrions avoir une fentre configure pour rendre OpenGL
			if (Initialize (&window, &keys) == FALSE)					// Initialisation de l'appel par utilisateur
			{
				// chec
				TerminateApplication (&window);							// Fermer la fentre, cela va grer la fermeture
			}
			else														// Autrement (Dmarrer le message pompe)
			{	// L'initialisation a t un succs
				isMessagePumpActive = TRUE;								// Mettre isMessagePumpActive  TRUE
				while (isMessagePumpActive == TRUE)						// Pendant que le message pompe est actif

				{
					// La fentre  t cre avec succs. Vrifier les messages Windows
					if (PeekMessage (&msg, window.hWnd, 0, 0, PM_REMOVE) != 0)
					{
						// Vrifier le message WM_QUIT
						if (msg.message != WM_QUIT)						// Le message est-il un message WM_QUIT?
						{
							DispatchMessage (&msg);						// Sinon, envoyez le message
						}
						else											// Sinon (si le message est WM_QUIT)
						{
							isMessagePumpActive = FALSE;				//Terminer le message pompe
						}
					}
					else												// S'il n'y a pas de messages
					{
						if (window.isVisible == FALSE)					// Si la fentre n'est pas visible
						{
							WaitMessage();								// Application Is Minimized Wait For A Message
						}
					}
				}														// Boucle tan que isMessagePumpActive == TRUE
			}															// Si (Initialize (...

			// L'application est termine
			Deinitialize ();											// Dfini par l'utilisateur DeInitialization

			DestroyWindowGL (&window);									// Dtruire la fentre active
		}
		else															// Si la cration de la fentre a chou
		{
			// Erreur de cration de fentre
			MessageBox (HWND_DESKTOP, "Erreur lors de la cration de la fentre OpenGL", "Erreur", MB_OK | MB_ICONEXCLAMATION);
			g_isProgramLooping = FALSE;									// Terminer la boucle
		}
	}																	// Tandis que (isProgramLooping)

	UnregisterClass (application.className, application.hInstance);		// UnRegister Classe de fentre
	return 0;
}																		// Fin de WinMain()
